@@ -1,6 +1,4 @@
 package MIME::Decoder::Base64;
-use strict;
-use warnings;
 
 
 =head1 NAME
@@ -30,15 +28,13 @@ of course).
 
 =item *
 
-When B<encoding>, the input is read 6840 (120 * 57) bytes at a time.
-Each section of 57 bytes is encoded as a line containing 76 Base64
-characters.
+When B<encoding>, the input is read 45 bytes at a time: this ensures
+that the output lines are not too long.   We chose 45 since it is
+a multiple of 3 and produces lines under 76 characters, as RFC-1521 
+specifies.
 
 =back
 
-=head1 SEE ALSO
-
-L<MIME::Decoder>
 
 =head1 AUTHOR
 
@@ -47,20 +43,27 @@ Eryq (F<eryq@zeegee.com>), ZeeGee Software Inc (F<http://www.zeegee.com>).
 All rights reserved.  This program is free software; you can redistribute 
 it and/or modify it under the same terms as Perl itself.
 
+
+=head1 VERSION
+
+$Revision: 6.107 $ $Date: 2003/06/06 23:41:39 $
+
 =cut
 
 use vars qw(@ISA $VERSION);
 use MIME::Decoder;
 use MIME::Base64 2.04;    
-use MIME::Tools qw(debug);
+use MIME::Tools qw(:msgs);
+use MIME::Tools::Utils qw(:msgs);
 
 @ISA = qw(MIME::Decoder);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.516";
+$VERSION = substr q$Revision: 6.107 $, 10;
 
-### How many bytes to encode at a time (must be a multiple of 3)
-my $EncodeChunkLength = 120 * 57;
+### How many bytes to encode at a time (must be a multiple of 3, and
+### less than (76 * 0.75)!
+my $EncodeChunkLength = 45;
 
 ### How many bytes to decode at a time?
 my $DecodeChunkLength = 32 * 1024;
@@ -75,7 +78,6 @@ sub decode_it {
     
     ### Create a suitable buffer:
     my $buffer = ' ' x (120 + $DecodeChunkLength); $buffer = '';
-    debug "in = $in; out = $out";
 
     ### Get chunks until done:
     local($_) = ' ' x $DecodeChunkLength;    
@@ -119,10 +121,9 @@ sub encode_it {
 
     my $nread;
     my $buf = '';
-    my $nl = $MIME::Entity::BOUNDARY_DELIMITER || "\n";
     while ($nread = $in->read($buf, $EncodeChunkLength)) {
-	$encoded = encode_base64($buf, $nl);
-	$encoded .= $nl unless ($encoded =~ /$nl\Z/);   ### ensure newline!
+	$encoded = encode_base64($buf);
+	$encoded .= "\n" unless ($encoded =~ /\n\Z/);   ### ensure newline!
 	$out->print($encoded);
     }
     1;
