@@ -20,23 +20,20 @@ Common non-standard MIME encodings for this:
     x-gzip64
 
 Since this class relies on external programs which may not
-exist on your machine, MIME-tools does not "install" it by default.
+exist on your machine, MIME-tools does not "install" it by default.  
 To use it, you need to say in your main program:
 
     install MIME::Decoder::Gzip64 'x-gzip64';
 
-Note: if this class isn't working for you, you may need to change the
-commands it runs.  In your main program, you can do so by setting up
-the two commands which handle the compression/decompression.
+Note: if this class isn't working for you, you may need to change
+the commands it runs.  In your main program, you can do so
+by setting up the two commands which 
 
     use MIME::Decoder::Gzip64;
-
+     
     $MIME::Decoder::Gzip64::GZIP   = 'gzip -c';
     $MIME::Decoder::Gzip64::GUNZIP = 'gzip -d -c';
 
-=head1 SEE ALSO
-
-L<MIME::Decoder>
 
 =head1 AUTHOR
 
@@ -46,22 +43,27 @@ All rights reserved.  This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
 
 
+=head1 VERSION
+
+$Revision: 6.107 $ $Date: 2003/06/06 23:41:40 $
+
 =cut
 
 
 require 5.002;
-use strict;
-use vars qw(@ISA $VERSION $GZIP $GUNZIP);
+use vars qw(@ISA $VERSION $ZIP $UNZIP);
 use MIME::Decoder;
 use MIME::Base64;
 use MIME::Decoder::Base64;
-use MIME::Tools qw(tmpopen whine);
+use MIME::Tools qw( $LOG);
+use MIME::Tools::Utils qw(tmpopen);
+use IO::Wrap;
 
 # Inheritance:
 @ISA = qw(MIME::Decoder::Base64);
 
 # The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.517";
+$VERSION = substr q$Revision: 6.107 $, 10;
 
 # How to compress stdin to stdout:
 $GZIP   = "gzip -c";
@@ -78,7 +80,7 @@ sub decode_it {
     my ($self, $in, $out) = @_;
 
     # Open a temp file (assume the worst, that this is a big stream):
-    my $tmp = tmpopen() || die "can't get temp file";
+    my $tmp = wraphandle(tmpopen() || die "can't get temp file");
 
     # Stage 1: decode the base64'd stream into zipped data:
     $self->SUPER::decode_it($in, $tmp)    or die "base64 decoding failed!";
@@ -94,10 +96,10 @@ sub decode_it {
 #
 sub encode_it {
     my ($self, $in, $out) = @_;
-    whine "Encoding ", $self->encoding, " is not standard MIME!"; 
+    $LOG->warning("Encoding ", $self->encoding, " is not standard MIME!"); 
     
     # Open a temp file (assume the worst, that this is a big stream):
-    my $tmp = tmpopen() || die "can't get temp file";
+    my $tmp = wraphandle(tmpopen() || die "can't get temp file");
   
     # Stage 1: zip the raw data:
     $self->filter($in, $tmp, $GZIP)       or die "gzip encoding failed!";
