@@ -1,6 +1,4 @@
 package MIME::Decoder::NBit;
-use strict;
-use warnings;
 
 
 =head1 NAME
@@ -76,10 +74,6 @@ domain.  The default action is to warn once per encoding if violations
 are detected; the warnings may be silenced with the QUIET configuration
 of L<MIME::Tools>.
 
-=head1 SEE ALSO
-
-L<MIME::Decoder>
-
 
 =head1 AUTHOR
 
@@ -88,17 +82,24 @@ Eryq (F<eryq@zeegee.com>), ZeeGee Software Inc (F<http://www.zeegee.com>).
 All rights reserved.  This program is free software; you can redistribute 
 it and/or modify it under the same terms as Perl itself.
 
+
+=head1 VERSION
+
+$Revision: 6.107 $ $Date: 2003/06/06 23:41:41 $
+
+
 =cut
 
 use vars qw(@ISA $VERSION);
 
 use MIME::Decoder;
 use MIME::Tools qw(:msgs);
+use MIME::Tools::Utils qw(:msgs);
 
 @ISA = qw(MIME::Decoder);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.516";
+$VERSION = substr q$Revision: 6.107 $, 10;
 
 ### How many bytes to decode at a time?
 my $DecodeChunkLength = 8 * 1024;
@@ -118,7 +119,7 @@ sub decode_it {
     while ($in->read($_, $DecodeChunkLength)) {
 	$and_also = $in->getline;
 	$_ .= $and_also if defined($and_also);
-
+	
 	### Just got a chunk ending in a line.
 	s/\015\012$/\n/g;
 	$out->print($_);
@@ -141,12 +142,14 @@ sub encode_it {
 
 	### Whine if encoding is 7bit and it has 8-bit data:
 	if ($seven_bit && ($line =~ /[\200-\377]/)) { ### oops! saw 8-bit data!
-	    whine "saw 8-bit data while encoding 7bit" unless $saw_8bit++;
+	    $LOG->warning("saw 8-bit data while encoding 7bit")
+		unless $saw_8bit++;
 	}
 
 	### Whine if long lines detected:
 	if (length($line) > 998) {
-	    whine "saw long line while encoding 7bit/8bit" unless $saw_long++;
+	    $LOG->warning("saw long line while encoding 7bit/8bit")
+		unless $saw_long++;
 	}
 
 	### Output!
