@@ -11,6 +11,8 @@ Before reading further, you should see L<MIME::Parser> to make sure that
 you understand where this module fits into the grand scheme of things.
 Go on, do it now.  I'll wait.
 
+Now read MIME::Parser::AbstractFiler.
+
 Ready?  Ok... now read L<"DESCRIPTION"> below, and everything else
 should make sense.
 
@@ -18,9 +20,9 @@ should make sense.
 =head2 Public interface
 
     ### Create a "filer" of the desired class:
-    my $filer = MIME::Parser::FileInto->new($dir);
-    my $filer = MIME::Parser::FileUnder->new($basedir);
-    ...
+    $filer = MIME::Parser::FlatFiler->new($dir);
+ (or)
+    $filer = MIME::Parser::TreeFiler->new($topdir);
 
     ### Want added security?  Don't let outsiders name your files:
     $filer->ignore_filename(1);
@@ -28,13 +30,14 @@ should make sense.
     ### Prepare for the parsing of a new top-level message:
     $filer->init_parse;
 
-    ### Return the path where this message's data should be placed:
+    ### Given the head of a new message, return the path where this
+    ### message's data should be placed:
     $path = $filer->output_path($head);
 
 
 =head2 Semi-public interface
 
-These methods might be overridden or ignored in some subclasses,
+These methods might be overriden or ignored in some subclasses,
 so they don't all make sense in all circumstances:
 
     ### Tweak the mapping from content-type to extension:
@@ -85,10 +88,10 @@ you would do this:
 
 =head2 Writing your own subclasses
 
-There are two standard "Filer" subclasses (see below):
-B<MIME::Parser::FileInto>, which throws all files from all parses
-into the same directory, and B<MIME::Parser::FileUnder> (preferred), which
-creates a subdirectory for each message.  Hopefully, these will be
+There are two standard "Filer" subclasses:
+B<MIME::Parser::FlatFiler>, which throws all files from all parses
+into the same directory, and B<MIME::Parser::TreeFiler> (preferred),
+which creates a subdirectory for each message.  Hopefully, these will be
 sufficient for most uses, but just in case...
 
 The only method you have to override is L<output_path()|/output_path>:
@@ -117,21 +120,16 @@ method and just override one of its components:
 
 =head1 PUBLIC INTERFACE
 
-
-=head2 MIME::Parser::Filer
-
-This is the abstract superclass of all "filer" objects.
-
 =over 4
 
 =cut
 
 use strict;
+use base qw(MIME::Parser::AbstractFiler);
 
 ### Kit modules:
-use MIME::Tools qw(:msgtypes);
+use MIME::Tools;
 use File::Spec;
-use File::Path qw(rmtree);
 use MIME::WordDecoder;
 
 ### Output path uniquifiers:
@@ -219,42 +217,29 @@ message/*                       .msg
 
 );
 
+
 #------------------------------
 
-=item new INITARGS...
-
-I<Class method, constructor.>
-Create a new outputter for the given parser.
-Any subsequent arguments are given to init(), which subclasses should
-override for their own use (the default init does nothing).
-
-=cut
-
-sub new {
-    my ($class, @initargs) = @_;
-    my $self = bless {
-	MPF_Prefix    => "msg",
-	MPF_Dir       => ".",
-	MPF_Ext       => { %DefaultTypeToExt },
-	MPF_Purgeable => [],       ### files created by the last parse
-
-	MPF_MaxName   => 80,       ### max filename before treated as evil
-	MPF_TrimRoot  => 14,       ### trim root to this length
-	MPF_TrimExt   => 3,        ### trim extension to this length
-    }, $class;
-    $self->init(@initargs);
-    $self;
-}
-
 sub init {
-    ### no-op
+    my $self = shift;
+
+    ### Always invoke inherited method first!
+    $self->SUPER::init(@_);
+
+    $self->{MPF_Prefix} = "msg";
+    $self->{MPF_Dir}    = ".";
+    $self->{MPF_Ext}    = { %DefaultTypeToExt };
+
+    $self->{MPF_MaxName} = 80;       ### max filename before treated as evil
+    $self->{MPF_TrimRoot} = 14;      ### trim root to this length
+    $self->{MPF_TrimExt}  = 3;       ### trim extension to this length
 }
 
 #------------------------------
 #
 # cleanup_dir
 #
-# Instance method, private.
+# Class method, utility.
 # Cleanup a directory, defaulting empty to "."
 #
 sub cleanup_dir {
@@ -267,72 +252,15 @@ sub cleanup_dir {
 
 #------------------------------
 
-=item results RESULTS
-
-I<Instance method.>
-Link this filer to a MIME::Parser::Results object which will
-tally the messages.  Notice that we avoid linking it to the
-parser to avoid circular reference!
-
-=cut
-
-sub results {
-    my ($self, $results) = @_;
-    $self->{MPF_Results} = $results if (@_ > 1);
-    $self->{MPF_Results};
-}
-
-### Log debug messages:
-sub debug {
-    my $self = shift;
-    if (MIME::Tools->debugging()) {
-	if ($self->{MPF_Results}) {
-	    unshift @_, $self->{MPF_Results}->indent;
-	    $self->{MPF_Results}->msg($M_DEBUG, @_);
-	}
-	MIME::Tools::debug(@_);
-    }
-}
-
-### Log warning messages:
-sub whine {
-    my $self = shift;
-    if ($self->{MPF_Results}) {
-	unshift @_, $self->{MPF_Results}->indent;
-	$self->{MPF_Results}->msg($M_WARNING, @_);
-    }
-    MIME::Tools::whine(@_);
-}
-
-#------------------------------
-
-=item init_parse
-
-I<Instance method.>
-Prepare to start parsing a new message.
-Subclasses should always be sure to invoke the inherited method.
-
-=cut
-
-sub init_parse {
-    my $self = shift;
-    $self->{MPF_Purgeable} = [];
-}
-
-#------------------------------
-
 =item evil_filename FILENAME
 
 I<Instance method.>
 Is this an evil filename; i.e., one which should not be used
 in generating a disk file name?  It is if any of these are true:
 
-    * it is empty or entirely whitespace
-    * it contains leading or trailing whitespace
+    * it is empty
     * it is a string of dots: ".", "..", etc.
-    * it contains characters not in the set: "A" - "Z", "a" - "z",
-      "0" - "9", "-", "_", "+", "=", ".", ",", "@", "#",
-      "$", and " ".
+    * it contains a known "path" character: '/' '\' ':' '[' ']'
     * it is too long
 
 If you just want to change this behavior, you should override
@@ -358,15 +286,15 @@ eye of the beholder.>
 sub evil_filename {
     my ($self, $name) = @_;
 
-    $self->debug("is this evil? '$name'");
+    $self->logger->debug("is this evil? '$name'");
 
     return 1 if (!defined($name) or ($name eq ''));   ### empty
-    return 1 if ($name =~ m{(^\s)|(\s+\Z)});  ### leading/trailing whitespace
     return 1 if ($name =~ m{^\.+\Z});         ### dots
-    return 1 if ($name =~ /[^-A-Z0-9_+=.,@\#\$\% ]/i); # Only allow good chars
+    return 1 if ($name =~ tr{\\/:[]}{});      ### path characters
     return 1 if ($self->{MPF_MaxName} and
 		 (length($name) > $self->{MPF_MaxName}));
-    $self->debug("it's ok");
+
+    $self->logger->debug("it's ok");
     0;
 }
 
@@ -396,15 +324,9 @@ sub exorcise_filename {
     my ($self, $fname) = @_;
 
     ### Isolate to last path element:
-    my $last = $fname;
-
-    ### Path separators are / or \
-    $last =~ s{^.*[/\\]}{};
-
-    ### Convert semi-evil characters to underscores
-    $last =~ s/[\/\\\[\]:]/_/g;
+    my $last = $fname; $last =~ s{^.*[/\\\[\]:]}{};
     if ($last and !$self->evil_filename($last)) {
-	$self->debug("looks like I can use the last path element");
+	$self->logger->debug("looks like I can use the last path element");
 	return $last;
     }
 
@@ -412,22 +334,12 @@ sub exorcise_filename {
     my ($root, $ext) = (($last =~ /^(.*)\.([^\.]+)\Z/)
 			? ($1, $2)
 			: ($last, ''));
-    ### Delete leading and trailing whitespace
-    $root =~ s/^\s+//;
-    $ext  =~ s/\s+$//;
     $root = substr($root, 0, ($self->{MPF_TrimRoot} || 14));
     $ext  = substr($ext,  0, ($self->{MPF_TrimExt}  ||  3));
     $ext =~ /^\w+$/ or $ext = "dat";
     my $trunc = $root . ($ext ? ".$ext" : '');
     if (!$self->evil_filename($trunc)) {
-	$self->debug("looks like I can use the truncated last path element");
-	return $trunc;
-    }
-
-    ### Remove all bad characters
-    $trunc =~ s/([^-A-Z0-9_+=.,@\#\$ ])/sprintf("%%%02X", unpack("C", $1))/ige;
-    if (!$self->evil_filename($trunc)) {
-	$self->debug("looks like I can use a munged version of the truncated last path element");
+	$self->logger->debug("looks like I can use a truncated last path element");
 	return $trunc;
     }
 
@@ -474,10 +386,11 @@ sub find_unused_path {
 	my $sname = $fname; $sname =~ s/^(.*?)(\.|\Z)/$1$suffix$2/;
 	my $path = File::Spec->catfile($dir, $sname);
 	if (! -e $path) {   ### it's good!
-	    $i and $self->whine("collision with $fname in $dir: using $path");
+	    $i and $self->logger->warning
+		("collision with $fname in $dir: using $path");
 	    return $path;
 	}
-	$self->debug("$path already taken");
+	$self->logger->debug("$path already taken");
     } continue { ++$i; }
 }
 
@@ -506,8 +419,9 @@ sub ignore_filename {
 
 =item output_dir HEAD
 
-I<Instance method.>
-Return the output directory for the given header.
+I<Instance method, for subclasses only.>
+Return the output directory for a singlepart entity with
+the given header.
 The default method returns ".".
 
 =cut
@@ -521,7 +435,7 @@ sub output_dir {
 
 =item output_filename HEAD
 
-I<Instance method, subclasses only.>
+I<Instance method, for subclasses only.>
 A given recommended filename was either not given, or it was judged
 to be evil.  Return a fake name, possibly using information in the
 message HEADer.  Note that this is just the filename, not the full path.
@@ -537,7 +451,7 @@ sub output_filename {
     my ($self, $head) = @_;
 
     ### Get the recommended name:
-    my $recommended = $head->recommended_filename;
+    my $recommended = unmime $head->recommended_filename;
 
     ### Get content type:
     my ($type, $subtype) = split m{/}, $head->mime_type; $subtype ||= '';
@@ -554,7 +468,7 @@ sub output_filename {
 	       $self->{MPF_Ext}{"*/*"} ||
 	       ".dat");
 
-    ### Get a prefix:
+    ### Get a prefix, and make up a name:
     ++$GFileNo;
     return ($self->output_prefix . "-$$-$GFileNo$ext");
 }
@@ -616,7 +530,7 @@ sub output_type_ext  {
 
 =item output_path HEAD
 
-I<Instance method, subclasses only.>
+I<Instance method, concrete override.>
 Given a MIME head for a file to be extracted, come up with a good
 output pathname for the extracted file.  This is the only method
 you need to worry about if you are building a custom filer.
@@ -662,16 +576,18 @@ sub output_path {
     ### Get the output directory:
     my $dir = $self->output_dir($head);
 
-    ### Get the output filename as UTF-8
-    my $fname = $head->recommended_filename;
+    ### Get the output filename, decoding into the local character set:
+    my $fname = unmime $head->recommended_filename;
 
     ### Can we use it:
     if    (!defined($fname)) {
-	$self->debug("no filename recommended: synthesizing our own");
+	$self->logger->debug
+	    ("no filename recommended: synthesizing our own");
 	$fname = $self->output_filename($head);
     }
     elsif ($self->ignore_filename) {
-	$self->debug("ignoring all external filenames: synthesizing our own");
+	$self->logger->debug
+	    ("ignoring all external filenames: synthesizing our own");
 	$fname = $self->output_filename($head);
     }
     elsif ($self->evil_filename($fname)) {
@@ -679,257 +595,34 @@ sub output_path {
 	### Can we save it by just taking the last element?
 	my $ex = $self->exorcise_filename($fname);
 	if (defined($ex) and !$self->evil_filename($ex)) {
-	    $self->whine("Provided filename '$fname' is regarded as evil, ",
-			 "but I was able to exorcise it and get something ",
-			 "usable.");
+	    $self->logger->warning
+		("Provided filename '$fname' is regarded as evil, ",
+		 "but I was able to exorcise it and get something ",
+		 "usable.");
 	    $fname = $ex;
 	}
 	else {
-	    $self->whine("Provided filename '$fname' is regarded as evil; ",
-			 "I'm ignoring it and supplying my own.");
+	    $self->logger->warning
+		("Provided filename '$fname' is regarded as evil; ",
+		 "I'm ignoring it and supplying my own.");
 	    $fname = $self->output_filename($head);
 	}
     }
-    $self->debug("planning to use '$fname'");
+    $self->logger->debug("planning to use '$fname'");
 
     ### Resolve collisions and return final path:
     return $self->find_unused_path($dir, $fname);
 }
 
-#------------------------------
-
-=item purge
-
-I<Instance method, final.>
-Purge all files/directories created by the last parse.
-This method simply goes through the purgeable list in reverse order
-(see L</purgeable>) and removes all existing files/directories in it.
-You should not need to override this method.
-
-=cut
-
-sub purge {
-    my ($self) = @_;
-    foreach my $path (reverse @{$self->{MPF_Purgeable}}) {
-	(-e $path) or next;   ### must check: might delete DIR before DIR/FILE
-	rmtree($path, 0, 1);
-	(-e $path) and $self->whine("unable to purge: $path");
-    }
-    1;
-}
-
-#------------------------------
-
-=item purgeable [FILE]
-
-I<Instance method, final.>
-Add FILE to the list of "purgeable" files/directories (those which
-will be removed if you do a C<purge()>).
-You should not need to override this method.
-
-If FILE is not given, the "purgeable" list is returned.
-This may be used for more-sophisticated purging.
-
-As a special case, invoking this method with a FILE that is an
-arrayref will replace the purgeable list with a copy of the
-array's contents, so [] may be used to clear the list.
-
-Note that the "purgeable" list is cleared when a parser begins a
-new parse; therefore, if you want to use purge() to do cleanup,
-you I<must> do so I<before> starting a new parse!
-
-=cut
-
-sub purgeable {
-    my ($self, $path) = @_;
-    return @{$self->{MPF_Purgeable}} if (@_ == 1);
-
-    if (ref($path)) { $self->{MPF_Purgeable} = [ @$path ]; }
-    else            { push @{$self->{MPF_Purgeable}}, $path; }
-    1;
-}
 
 =back
 
 =cut
 
 
-#------------------------------------------------------------
-#------------------------------------------------------------
-
-=head2 MIME::Parser::FileInto
-
-This concrete subclass of MIME::Parser::Filer supports filing
-into a given directory.
-
-=over 4
-
-=cut
-
-package MIME::Parser::FileInto;
-
-use strict;
-use vars qw(@ISA);
-@ISA = qw(MIME::Parser::Filer);
-
-#------------------------------
-
-=item init DIRECTORY
-
-I<Instance method, initiallizer.>
-Set the directory where all files will go.
-
-=cut
-
-sub init {
-    my ($self, $dir) = @_;
-    $self->{MPFI_Dir} = $self->cleanup_dir($dir);
-}
-
-#------------------------------
-#
-# output_dir HEAD
-#
-# I<Instance method, concrete override.>
-# Return the output directory where the files go.
-#
-sub output_dir {
-    shift->{MPFI_Dir};
-}
-
-=back
-
-=cut
-
-
-
-
-#------------------------------------------------------------
-#------------------------------------------------------------
-
-=head2 MIME::Parser::FileUnder
-
-This concrete subclass of MIME::Parser::Filer supports filing under
-a given directory, using one subdirectory per message, but with
-all message parts in the same directory.
-
-=over 4
-
-=cut
-
-package MIME::Parser::FileUnder;
-
-use strict;
-use vars qw(@ISA);
-@ISA = qw(MIME::Parser::Filer);
-
-#------------------------------
-
-=item init BASEDIR, OPTSHASH...
-
-I<Instance method, initiallizer.>
-Set the base directory which will contain the message directories.
-If used, then each parse of begins by creating a new subdirectory
-of BASEDIR where the actual parts of the message are placed.
-OPTSHASH can contain the following:
-
-=over 4
-
-=item DirName
-
-Explicitly set the name of the subdirectory which is created.
-The default is to use the time, process id, and a sequence number,
-but you might want a predictable directory.
-
-=item Purge
-
-Automatically purge the contents of the directory (including all
-subdirectories) before each parse.  This is really only needed if
-using an explicit DirName, and is provided as a convenience only.
-Currently we use the 1-arg form of File::Path::rmtree; you should
-familiarize yourself with the caveats therein.
-
-=back
-
-The output_dir() will return the path to this message-specific directory
-until the next parse is begun, so you can do this:
-
-    use File::Path;
-
-    $parser->output_under("/tmp");
-    $ent = eval { $parser->parse_open($msg); };   ### parse
-    if (!$ent) {	 ### parse failed
-	rmtree($parser->output_dir);
-	die "parse failed: $@";
-    }
-    else {               ### parse succeeded
-	...do stuff...
-    }
-
-=cut
-
-sub init {
-    my ($self, $basedir, %opts) = @_;
-
-    $self->{MPFU_Base}    = $self->cleanup_dir($basedir);
-    $self->{MPFU_DirName} = $opts{DirName};
-    $self->{MPFU_Purge}   = $opts{Purge};
-}
-
-#------------------------------
-#
-# init_parse
-#
-# I<Instance method, override.>
-# Prepare to start parsing a new message.
-#
-sub init_parse {
-    my $self = shift;
-
-    ### Invoke inherited method first!
-    $self->SUPER::init_parse;
-
-    ### Determine the subdirectory of their base to use:
-    my $subdir = (defined($self->{MPFU_DirName})
-		  ?       $self->{MPFU_DirName}
-		  :       ("msg-".scalar(time)."-$$-".$GSubdirNo++));
-    $self->debug("subdir = $subdir");
-
-    ### Determine full path to the per-message output directory:
-    $self->{MPFU_Dir} = File::Spec->catfile($self->{MPFU_Base}, $subdir);
-
-    ### Remove and re-create the per-message output directory:
-    rmtree $self->output_dir if $self->{MPFU_Purge};
-    (-d $self->output_dir) or
-	mkdir $self->output_dir, 0700 or
-	    die "mkdir ".$self->output_dir.": $!\n";
-
-    ### Add the per-message output directory to the puregables:
-    $self->purgeable($self->output_dir);
-    1;
-}
-
-#------------------------------
-#
-# output_dir HEAD
-#
-# I<Instance method, concrete override.>
-# Return the output directory that we used for the last parse.
-#
-sub output_dir {
-    shift->{MPFU_Dir};
-}
-
-=back
-
-=cut
-
 1;
 __END__
 
-=head1 SEE ALSO
-
-L<MIME::Tools>, L<MIME::Parser>
 
 =head1 AUTHOR
 
@@ -938,3 +631,8 @@ Eryq (F<eryq@zeegee.com>), ZeeGee Software Inc (F<http://www.zeegee.com>).
 All rights reserved.  This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
 
+
+=head1 VERSION
+
+$Revision: 6.106 $
+
