@@ -12,21 +12,25 @@ don't need to be looking at it at all.  But just in case...
 
     ### Create a top-level reader, where chunks end at EOF:
     $rdr = MIME::Parser::Reader->new();
-
+      
     ### Spawn a child reader, where chunks also end at a boundary:
     $subrdr = $rdr->spawn->add_boundary($bound);
-
+     
     ### Spawn a child reader, where chunks also end at a given string:
     $subrdr = $rdr->spawn->add_terminator($string);
-
+     
     ### Read until boundary or terminator:
     $subrdr->read_chunk($in, $out);
+    
+    ### Get info on how the stream ended:
+    $eos      = $subrdr->eos;
+    $eos_type = $subrdr->eos_type;
 
 
 =head1 DESCRIPTION
 
 A line-oriented reader which can deal with virtual end-of-stream
-defined by a collection of boundaries.
+defined by a collection of boundaries. 
 
 B<Warning:> this is a private class solely for use by MIME::Parser.
 This class has no official public interface
@@ -34,6 +38,9 @@ This class has no official public interface
 =cut
 
 use strict;
+use IO::ScalarArray;
+use MIME::Tools qw(:msgs);
+use MIME::Tools::Utils qw(:msgs);
 
 ### All possible end-of-line sequences.
 ### Note that "" is included because last line of stream may have no newline!
@@ -66,7 +73,7 @@ sub new {
 # spawn
 #
 # I<Instance method.>
-# Return a reader which is mostly a duplicate, except that the EOS
+# Return a reader which is mostly a duplicate, except that the EOS 
 # accumulator is shared.
 #
 sub spawn {
@@ -125,7 +132,7 @@ sub has_bounds {
 # depth
 #
 # I<Instance method.>
-# How many levels are there?
+# How many levels are there? 
 #
 sub depth {
     scalar(@{shift->{Bounds}});
@@ -153,7 +160,7 @@ sub eos {
 # Return the high-level type of the given token (defaults to our token).
 #
 #    DELIM       saw an innermost boundary like --xyz
-#    CLOSE       saw an innermost boundary like --xyz--
+#    CLOSE       saw an innermost boundary like --xyz-- 
 #    DONE        callback returned false
 #    EOF         end of file
 #    EXT         saw boundary of some higher-level
@@ -169,7 +176,7 @@ sub eos_type {
 	return (($2 eq $self->{Bounds}[0]) ? $1 : 'EXT');
     }
     else {
-	die("internal error: unable to classify boundary token ($eos)");
+	internal_error "unable to classify boundary token ($eos)";
     }
 }
 
@@ -183,8 +190,8 @@ sub eos_type {
 #
 sub native_handle {
     my $fh = shift;
-    return $fh if $fh->isa('IO::File');
-    return $fh if (ref $fh eq 'GLOB');
+    return $fh  if $fh->isa('IO::File');
+    return $$fh if ($fh->isa('IO::Wrap') && (ref($$fh) eq 'GLOB'));
     undef;
 }
 
@@ -193,7 +200,7 @@ sub native_handle {
 # read_chunk INHANDLE, OUTHANDLE
 #
 # I<Instance method.>
-# Get lines until end-of-stream.
+# Get lines until [virtual] end of stream.
 # Returns the terminating-condition token:
 #
 #    DELIM xyz   saw boundary line "--xyz"
@@ -202,11 +209,11 @@ sub native_handle {
 #    EOF         end of file
 
 # Parse up to (and including) the boundary, and dump output.
-# Follows the RFC 2046 specification, that the CRLF immediately preceding
+# Follows the RFC-1521 specification, that the CRLF immediately preceding 
 # the boundary is part of the boundary, NOT part of the input!
 #
-# NOTE: while parsing bodies, we take care to remember the EXACT end-of-line
-# sequence.  This is because we *may* be handling 'binary' encoded data, and
+# NOTE: while parsing, we take care to remember the EXACT end-of-line
+# sequence.  This is because we *may* be handling 'binary' encoded data, and 
 # in that case we can't just massage \r\n into \n!  Don't worry... if the
 # data is styled as '7bit' or '8bit', the "decoder" will massage the CRLF
 # for us.  For now, we're just trying to chop up the data stream.
@@ -217,13 +224,8 @@ sub native_handle {
 # last.  I strip the last CRLF when I hit the boundary.
 
 sub read_chunk {
-    my ($self, $in, $out, $keep_newline, $normalize_newlines) = @_;
-
-    # If we're parsing a preamble or epilogue, we need to keep the blank line
-    # that precedes the boundary line.
-    $keep_newline ||= 0;
-
-    $normalize_newlines ||= 0;
+    my ($self, $in, $out) = @_;
+    
     ### Init:
     my %bh = %{$self->{BH}};
     my %th = %{$self->{TH}}; my $thx = keys %th;
@@ -231,7 +233,7 @@ sub read_chunk {
     my $maybe;
     my $last = '';
     my $eos  = '';
-
+    
     ### Determine types:
     my $n_in  = native_handle($in);
     my $n_out = native_handle($out);
@@ -240,60 +242,50 @@ sub read_chunk {
     if ($n_in) {
 	if ($n_out) {            ### native input, native output [fastest]
 	    while (<$n_in>) {
-		# Normalize line ending
-		$_ =~ s/(:?\n\r|\r\n|\r)$/\n/ if $normalize_newlines;
 		if (substr($_, 0, 2) eq '--') {
 		    ($maybe = $_) =~ s/[ \t\r\n]+\Z//;
 		    $bh{$maybe} and do { $eos = $bh{$maybe}; last };
 		}
 		$thx and $th{$_} and do { $eos = $th{$_}; last };
-		print $n_out $last; $last = $_;
+		print $n_out $last; $last = $_; 
 	    }
 	}
 	else {                   ### native input, OO output [slower]
-	    while (<$n_in>) {
-		# Normalize line ending
-		$_ =~ s/(:?\n\r|\r\n|\r)$/\n/ if $normalize_newlines;
+	    while (<$n_in>) { 
 		if (substr($_, 0, 2) eq '--') {
 		    ($maybe = $_) =~ s/[ \t\r\n]+\Z//;
 		    $bh{$maybe} and do { $eos = $bh{$maybe}; last };
 		}
 		$thx and $th{$_} and do { $eos = $th{$_}; last };
-		$out->print($last); $last = $_;
+		$out->print($last); $last = $_; 
 	    }
 	}
     }
     else {
 	if ($n_out) {            ### OO input, native output [even slower]
-	    while (defined($_ = $in->getline)) {
-		# Normalize line ending
-		$_ =~ s/(:?\n\r|\r\n|\r)$/\n/ if $normalize_newlines;
+	    while (defined($_ = $in->getline)) { 
 		if (substr($_, 0, 2) eq '--') {
 		    ($maybe = $_) =~ s/[ \t\r\n]+\Z//;
 		    $bh{$maybe} and do { $eos = $bh{$maybe}; last };
 		}
 		$thx and $th{$_} and do { $eos = $th{$_}; last };
-		print $n_out $last; $last = $_;
+		print $n_out $last; $last = $_;  
 	    }
 	}
 	else {                   ### OO input, OO output [slowest]
-	    while (defined($_ = $in->getline)) {
-		# Normalize line ending
-		$_ =~ s/(:?\n\r|\r\n|\r)$/\n/ if $normalize_newlines;
+	    while (defined($_ = $in->getline)) { 
 		if (substr($_, 0, 2) eq '--') {
 		    ($maybe = $_) =~ s/[ \t\r\n]+\Z//;
 		    $bh{$maybe} and do { $eos = $bh{$maybe}; last };
 		}
 		$thx and $th{$_} and do { $eos = $th{$_}; last };
-		$out->print($last); $last = $_;
+		$out->print($last); $last = $_; 
 	    }
 	}
     }
-
-    # Write out last held line, removing terminating CRLF if ended on bound,
-    # unless the line consists only of CRLF and we're wanting to keep the
-    # preceding blank line (as when parsing a preamble)
-    $last =~ s/[\r\n]+\Z// if ($eos =~ /^(DELIM|CLOSE)/ && !($keep_newline && $last =~ m/^[\r\n]\z/));
+    
+    ### Write out last held line, removing terminating CRLF if ended on bound:
+    $last =~ s/[\r\n]+\Z// if ($eos =~ /^(DELIM|CLOSE)/);
     $out->print($last);
 
     ### Save and return what we finished on:
@@ -307,22 +299,14 @@ sub read_chunk {
 #
 # I<Instance method.>
 # Read lines into the given array.
-#
+# 
 sub read_lines {
     my ($self, $in, $outlines) = @_;
-
-    my $data = '';
-    open(my $fh, '>', \$data) or die $!;
-    $self->read_chunk($in, $fh);
-    @$outlines =  split(/^/, $data);
-    close $fh;
-
+    $self->read_chunk($in, IO::ScalarArray->new($outlines));
+    shift @$outlines if ($outlines->[0] eq '');   ### leading empty line
     1;
 }
 
 1;
 __END__
 
-=head1 SEE ALSO
-
-L<MIME::Tools>, L<MIME::Parser>
