@@ -1,11 +1,10 @@
 package MIME::WordDecoder;
 
+
 =head1 NAME
 
-MIME::WordDecoder - decode RFC 2047 encoded words to a local representation
+MIME::WordDecoder - decode RFC-1522 encoded words to a local representation
 
-WARNING: Most of this module is deprecated and may disappear.  The only
-function you should use for MIME decoding is "mime_to_perl_string".
 
 =head1 SYNOPSIS
 
@@ -19,7 +18,7 @@ See L<"DESCRIPTION"> for how this class works.
     $wd = default MIME::WordDecoder;
 
     ### Get a word-decoder which maps to ISO-8859-1 (Latin1):
-    $wd = supported MIME::WordDecoder "ISO-8859-1";
+    $wd = MIME::WordDecoder->for("ISO-8859-1");
 
 
     ### Decode a MIME string (e.g., into Latin1) via the default decoder:
@@ -28,16 +27,9 @@ See L<"DESCRIPTION"> for how this class works.
     ### Decode a string using the default decoder, non-OO style:
     $str = unmime('To: =?ISO-8859-1?Q?Keld_J=F8rn_Simonsen?= <keld>');
 
-    ### Decode a string to an internal Perl string, non-OO style
-    ### The result is likely to have the UTF8 flag ON.
-    $str = mime_to_perl_string('To: =?ISO-8859-1?Q?Keld_J=F8rn_Simonsen?= <keld>');
 
 =head1 DESCRIPTION
 
-WARNING: Most of this module is deprecated and may disappear.  It
-duplicates (badly) the function of the standard 'Encode' module.  The
-only function you should rely on is mime_to_perl_string.
-
 A MIME::WordDecoder consists, fundamentally, of a hash which maps
 a character set name (US-ASCII, ISO-8859-1, etc.) to a subroutine which
 knows how to take bytes in that character set and turn them into
@@ -72,15 +64,7 @@ Here's a decoder which uses that:
 
    ### ...which will now hold: "To: Keld J#rn Simonsen <keld>"
 
-The UTF-8 built-in decoder decodes everything into Perl's internal
-string format, possibly turning on the internal UTF8 flag.  Use it like
-this:
 
-    $wd = supported MIME::WordDecoder 'UTF-8';
-    $perl_string = $wd->decode('To: =?ISO-8859-1?Q?Keld_J=F8rn_Simonsen?= <keld>');
-    # perl_string will be a valid UTF-8 string with the "UTF8" flag set.
-
-Generally, you should use the UTF-8 decoder in preference to "unmime".
 
 =head1 PUBLIC INTERFACE
 
@@ -89,13 +73,14 @@ Generally, you should use the UTF-8 decoder in preference to "unmime".
 =cut
 
 use strict;
+use MIME::Tools;
 use Carp qw( carp croak );
 use MIME::Words qw(decode_mimewords);
 use Exporter;
 use vars qw(@ISA @EXPORT);
-
+   
 @ISA = qw(Exporter);
-@EXPORT = qw( unmime mime_to_perl_string );
+@EXPORT = qw( unmime );
 
 
 
@@ -120,8 +105,6 @@ my %Handler =
 ### Global default decoder.  We init it below.
 my $Default;
 
-### Global UTF8 decoder.
-my $DefaultUTF8;
 
 #------------------------------
 
@@ -130,6 +113,9 @@ my $DefaultUTF8;
 I<Class method.>
 Get/set the default DECODER object.
 
+B<Note:> as of 6.x, this is initiallized to a decoder for the 
+default charset in this locale -- I<not> for ISO-8859-1.
+
 =cut
 
 sub default {
@@ -142,28 +128,43 @@ sub default {
 
 #------------------------------
 
-=item supported CHARSET, [DECODER]
+=item for CHARSET, [DECODER]
 
 I<Class method.>
 If just CHARSET is given, returns a decoder object which maps
 data into that character set (the character set is forced to
 all-uppercase).
 
-    $wd = supported MIME::WordDecoder "ISO-8859-1";
+    $wd = MIME::WordDecoder->for("ISO-8859-1") || die "unsupported";
 
-If DECODER is given, installs such an object:
+If DECODER is given, installs such an object, which then
+overrides whatever for() would normally return.
 
-    MIME::WordDecoder->supported("ISO-8859-1" =>
-				 (new MIME::WordDecoder::ISO_8859 "1"));
+    MIME::WordDecoder->for("ISO-8859-1" =>
+			   (new MIME::WordDecoder::ISO_8859 "1"));
 
 You should not override this method.
 
 =cut
 
-sub supported {
+sub for {
     my ($class, $charset, $decoder) = @_;
-    $DecoderFor{uc($charset)} = $decoder if (@_ > 2);
-    $DecoderFor{uc($charset)};
+    $charset = uc($charset);
+    
+    ### Install an overriding decoder?
+    if (@_ > 2) {
+	$DecoderFor{uc($charset)} = $decoder;
+    }
+
+    ### Return a suitable decoder, creating/cacheing if needed:
+    $DecoderFor{$charset} ||=
+	MIME::WordDecoder::Unicode->new($charset);
+}
+
+### Backcompat:
+sub supported {
+    my $class = shift;
+    $class->for(@_);
 }
 
 #------------------------------
@@ -171,8 +172,8 @@ sub supported {
 =item new [\@HANDLERS]
 
 I<Class method, constructor.>
-If \@HANDLERS is given, then @HANDLERS is passed to handler()
-to initialize the internal map.
+If \@HANDLERS is given, then @HANDLERS is passed to _handler()
+to initiallize the internal map.
 
 =cut
 
@@ -181,7 +182,7 @@ sub new {
     my $self = bless { MWD_Map=>{} }, $class;
 
     ### Init the map:
-    $self->handler(@$h);
+    $self->_handler(@$h);
 
     ### Add fallbacks:
     $self->{MWD_Map}{'*'}   ||= $Handler{WARN};
@@ -191,63 +192,6 @@ sub new {
 
 #------------------------------
 
-=item handler CHARSET=>\&SUBREF, ...
-
-I<Instance method.>
-Set the handler SUBREF for a given CHARSET, for as many pairs
-as you care to supply.
-
-When performing the translation of a MIME-encoded string, a
-given SUBREF will be invoked when translating a block of text
-in character set CHARSET.  The subroutine will be invoked with
-the following arguments:
-
-    DATA    - the data in the given character set.
-    CHARSET - the upcased character set name, which may prove useful
-              if you are using the same SUBREF for multiple CHARSETs.
-    DECODER - the decoder itself, if it contains configuration information
-              that your handler function needs.
-
-For example:
-
-    $wd = new MIME::WordDecoder;
-    $wd->handler('US-ASCII'   => "KEEP");
-    $wd->handler('ISO-8859-1' => \&handle_latin1,
-		 'ISO-8859-2' => \&handle_latin1,
-		 '*'          => "DIE");
-
-Notice that, much as with %SIG, the SUBREF can also be taken from
-a set of special keywords:
-
-   KEEP     Pass data through unchanged.
-   IGNORE   Ignore data in this character set, without warning.
-   WARN     Ignore data in this character set, with warning.
-   DIE      Fatal exception with "can't handle character set" message.
-
-The subroutine for the special CHARSET of 'raw' is used for raw
-(non-MIME-encoded) text, which is supposed to be US-ASCII.
-The handler for 'raw' defaults to whatever was specified for 'US-ASCII'
-at the time of construction.
-
-The subroutine for the special CHARSET of '*' is used for any
-unrecognized character set.  The default action for '*' is WARN.
-
-=cut
-
-sub handler {
-    my $self = shift;
-
-    ### Copy the hash, and edit it:
-    while (@_) {
-	my $c   = shift;
-	my $sub = shift;
-	$self->{MWD_Map}{$c} = $self->real_handler($sub);
-    }
-    $self;
-}
-
-#------------------------------
-
 =item decode STRING
 
 I<Instance method.>
@@ -272,10 +216,79 @@ sub decode {
 	my $subr = $self->{MWD_Map}{$charset} || $self->{MWD_Map}{'*'};
 
 	### Map this chunk:
+	###print STDERR "$subr($data, $charset, $self)\n";
 	&$subr($data, $charset, $self);
     } decode_mimewords($str));
 }
 
+#------------------------------
+
+=item _handler CHARSET=>\&SUBREF, ...
+
+I<Instance method, initialization.>
+Set the handler SUBREF for a given CHARSET, for as many pairs
+as you care to supply.  Intended for subclass init only!
+
+When performing the translation of a MIME-encoded string, a
+given SUBREF will be invoked when translating a block of text
+in character set CHARSET.  The subroutine will be invoked with
+the following arguments:
+
+    DATA    - the data in the given character set.
+    CHARSET - the upcased character set name, which may prove useful
+              if you are using the same SUBREF for multiple CHARSETs.
+    DECODER - the decoder itself, if it contains configuration information
+              that your handler function needs.
+
+For example:
+
+    package MyDecoder;
+     
+    use base qw(MIME::WordDecoder);
+    
+    sub new {
+        my $class = shift;  
+        my $self = $class->SUPER::new();
+
+	$self->_handler('US-ASCII'   => "KEEP");
+	$self->_handler('ISO-8859-1' => \&handle_latin1,
+			'ISO-8859-2' => \&handle_latin2,
+			'*'          => "DIE");
+    }
+
+Notice that, much as with %SIG, the SUBREF can also be taken from
+a set of special keywords:
+
+   KEEP     Pass data through unchanged.
+   IGNORE   Ignore data in this character set, without warning.
+   WARN     Ignore data in this character set, with warning.
+   DIE      Fatal exception with "can't handle character set" message.
+
+The subroutine for the special CHARSET of 'raw' is used for raw
+(non-MIME-encoded) text, which is supposed to be US-ASCII.
+The handler for 'raw' defaults to whatever was specified for 'US-ASCII'
+at the time of construction.
+
+The subroutine for the special CHARSET of '*' is used for any
+unrecognized character set.  The default action for '*' is WARN.
+
+=cut
+
+sub _handler {
+    my $self = shift;
+
+    ### Copy the hash, and edit it:
+    while (@_) {
+	my $c   = shift;
+	my $sub = shift;
+	$self->{MWD_Map}{$c} = $self->real_handler($sub);
+    }
+    $self;
+}
+sub handler {
+    shift->_handler(@_);
+}
+
 #------------------------------
 #
 # guess_handler CHARSET
@@ -312,9 +325,6 @@ I<Function, exported.>
 Decode the given STRING using the default() decoder.
 See L<default()|/default>.
 
-You should consider using the UTF-8 decoder instead.  It decodes
-MIME strings into Perl's internal string format.
-
 =cut
 
 sub unmime($) {
@@ -322,21 +332,6 @@ sub unmime($) {
     $Default->decode($str);
 }
 
-=item mime_to_perl_string
-
-I<Function, exported.>
-Decode the given STRING into an internal Perl Unicode string.
-You should use this function in preference to all others.
-
-The result of mime_to_perl_string is likely to have Perl's
-UTF8 flag set.
-
-=cut
-
-sub mime_to_perl_string($) {
-    my $str = shift;
-    $DecoderFor{'UTF-8'}->decode($str);
-}
 
 =back
 
@@ -355,8 +350,160 @@ sub mime_to_perl_string($) {
 #------------------------------------------------------------
 #------------------------------------------------------------
 
+=item MIME::WordDecoder::Unicode
+
+A decoder which converts to and from Unicode, and
+in theory can handle anything.
+
+    ### Constructs a decoder which maps to cp1250:
+    $wd = new MIME::WordDecoder::Unicode "cp1250";
+
+    ### Decode anything into cp1250 (there's no slash-o, so you get "Jrn"):
+    $str = $wd->decode('To: =?ISO-8859-1?Q?Keld_J=F8rn_Simonsen?= <keld>');
+
+Unrecognized characters are simply omitted (sorry, but
+that's the way Unicode::Map works).  
+
+
+=cut
+
+package MIME::WordDecoder::Unicode;
+
+use strict;
+use vars qw(@ISA);
+@ISA = qw( MIME::WordDecoder );
+
+use Unicode::Map;
+use Unicode::String qw(utf8 utf16);
+
+
+#------------------------------
+# Class method, constructor.
+#
+sub new {
+    my ($class, $to_charset) = @_;
+
+    ### Fixup names:
+    $to_charset = normalize_charset($to_charset) ||
+	die "can't create mime-word decoder: unsupported charset \"$to_charset\"\n";  ### gick
+
+    ### Build it:
+    my $self = $class->SUPER::new();
+    $self->_handler('raw'       => \&h_map_raw,
+		    $to_charset => 'KEEP',
+		    '*'         => \&h_map_any);
+
+    $self->{MWDVUM_Maps} = {};
+    $self->{MWDVUM_ToCharset} = $to_charset;    
+    $self;
+}
+
+#------------------------------
+# Function.
+# Upcase and normalize the given charset to something Unicode::Map can handle.
+# Returns empty string if we really think we can't handle it.
+#
+# Special cases: UTF-8, UTF-16, UCS-2.
+#
+my %NormalizedCharset;
+sub normalize_charset {
+    my ($charset) = @_;
+
+    ### Cached?
+    return $NormalizedCharset{$charset}
+    if defined($NormalizedCharset{$charset});    
+
+    ### Nope: gotta figure it out:
+    my $uc = uc($charset);
+    if    ($uc =~ /^UTF[_-]?8$/) {
+	return $NormalizedCharset{$charset} = 'UTF-8'
+    }
+    elsif ($uc =~ /^((UTF[_-]?16)|(UCS[_-]?2))$/) {
+	return $NormalizedCharset{$charset} = 'UTF-16'
+    }
+    elsif (eval { Unicode::Map->new($uc) }) {
+	return $NormalizedCharset{$charset} = $uc;
+    }
+    else {
+	if ($uc =~ /^(ISO|CP)(.*)/){
+	    my ($std, $num) = ($1, $2); $num =~ s/^[-_]//;
+	    foreach my $try ("$std$num","$std-$num","${std}_$num") {
+		if (Unicode::Map->new($try)) {
+		    print STDERR "NORMALIZED $charset TO $try\n";
+		    return $NormalizedCharset{$charset} = $try;
+		}
+	    }
+	}
+    }
+    ### Outta luck:
+    return $NormalizedCharset{$charset} = '';
+}
+
+#------------------------------
+# Instance method.
+# Loads and returns the needed map.
+#
+sub umap {
+    my ($self, $charset) = @_;
+    $self->{MWDVUM_Maps}{$charset} ||= 
+	Unicode::Map->new($charset);
+}
+
+#------------------------------
+# Function, handler.
+# Maps raw data.
+#
+sub h_map_raw {
+    my ($data, $data_charset, $self) = @_;
+    h_map_any($data, 'us-ascii', $self);
+}
+
+#------------------------------
+# Function, handler.
+# Maps data with explicit charset.
+#
+sub h_map_any {
+    my ($data, $raw_data_charset, $self) = @_;
+
+    ### Normalize charset, and map to empty string if unhandled:
+    my $data_charset = normalize_charset($raw_data_charset);
+    if (!$data_charset) {
+	$MIME::Tools::LOG->warning("can't decode mime-words: unsupported charset \"$raw_data_charset\"");
+	return '';    ### silent
+    }
+
+    ### Map to utf16:
+    my $utf16;
+    if    ($data_charset eq 'UTF-16') {
+	$utf16 = $data;
+    }
+    elsif ($data_charset eq 'UTF-8') {
+	$utf16 = utf8($data)->utf16;
+    }
+    else {
+	$utf16 = $self->umap($data_charset)->to_unicode($data);
+    }    
+
+    ### Map to target charset:
+    if    ($self->{MWDVUM_ToCharset} eq 'UTF-16') {
+	return $utf16;
+    }
+    elsif ($self->{MWDVUM_ToCharset} eq 'UTF-8') {
+	return utf16($utf16)->utf8;
+    }
+    else {
+	return $self->umap($self->{MWDVUM_ToCharset})->from_unicode($utf16);
+    }
+}
+
+#------------------------------------------------------------
+#------------------------------------------------------------
+
 =item MIME::WordDecoder::ISO_8859
 
+I<Obsolete as of 6.x.> 
+See L</"MIME::WordDecoder::Unicode">.
+
 A simple decoder which keeps US-ASCII and the 7-bit characters
 of ISO-8859 character sets and UTF8, and also keeps 8-bit
 characters from the indicated character set.
@@ -425,7 +572,7 @@ sub h_keep7bit {
     $_;
 }
 
-### Note: should use Unicode::String, converting/manipulating
+### Note: should use Unicode::Map, converting/manipulating
 ### everything into full Unicode form.
 
 ### Keep 7bit UTF8 characters (ASCII).
@@ -435,9 +582,8 @@ sub h_utf8 {
     local $_    = $_[0];
 #   my $unknown = $_[2]->{MWDI_Unknown};
     my $latin1 = ($_[2]->{MWDI_Num} == 1);
-    #print STDERR "UTF8 in:  <$_>\n";
+    print STDERR "UTF8 in:  <$_>\n";
 
-    local($1,$2,$3);
     my $tgt = '';
     while (m{\G(
           ([\x00-\x7F])                | # 0xxxxxxx
@@ -452,7 +598,7 @@ sub h_utf8 {
         else                           { $tgt .= "\x00" }
     }
 
-    #print STDERR "UTF8 out: <$tgt>\n";
+    print STDERR "UTF8 out: <$tgt>\n";
     $tgt;
 }
 
@@ -463,9 +609,8 @@ sub h_utf16 {
     local $_    = $_[0];
 #   my $unknown = $_[2]->{MWDI_Unknown};
     my $latin1 = ($_[2]->{MWDI_Num} == 1);
-    #print STDERR "UTF16 in:  <$_>\n";
+    print STDERR "UTF16 in:  <$_>\n";
 
-    local($1,$2,$3,$4,$5);
     my $tgt = '';
     while (m{\G(
 		(  \x00  ([\x00-\x7F])) |  # 00000000 0xxxxxxx
@@ -479,7 +624,7 @@ sub h_utf16 {
         else                           { $tgt .= "\x00" }
     }
 
-    #print STDERR "UTF16 out: <$tgt>\n";
+    print STDERR "UTF16 out: <$tgt>\n";
     $tgt;
 }
 
@@ -498,8 +643,8 @@ sub new {
     my ($class, $num) = @_;
 
     my $self = $class->SUPER::new();
-    $self->handler('raw'      => 'KEEP',
-		   'US-ASCII' => 'KEEP');
+    $self->_handler('raw'      => 'KEEP',
+		    'US-ASCII' => 'KEEP');
 
     $self->{MWDI_Num} = $num;
     $self->{MWDI_Unknown} = "?";
@@ -596,87 +741,87 @@ sub decode {
 
 =cut
 
-package MIME::WordDecoder::UTF_8;
-use strict;
-use Encode qw();
-use Carp qw( carp );
-use vars qw(@ISA);
-
-@ISA = qw( MIME::WordDecoder );
-
-sub h_convert_to_utf8
-{
-	my ($data, $charset, $decoder) = @_;
-	$charset = 'US-ASCII' if ($charset eq 'raw');
-	my $enc = Encode::find_encoding($charset);
-	if (!$enc) {
-		carp "Unable to convert text in character set `$charset' to UTF-8... ignoring\n";
-		return '';
-	}
-	my $ans = $enc->decode($data, Encode::FB_PERLQQ);
-	return $ans;
-}
-
-sub new {
-	my ($class) = @_;
-	my $self = $class->SUPER::new();
-	$self->handler('*'     => \&h_convert_to_utf8);
-}
-
-
 #------------------------------------------------------------
 #------------------------------------------------------------
 
 package MIME::WordDecoder;
 
-### Now we can init the default handler.
-$Default = (MIME::WordDecoder::ISO_8859->new('1'));
-
-
-### Add US-ASCII handler:
-$DecoderFor{"US-ASCII"} = MIME::WordDecoder::US_ASCII->new;
-
-### Add ISO-8859-{1..15} handlers:
-for (1..15) {
-    $DecoderFor{"ISO-8859-$_"} = MIME::WordDecoder::ISO_8859->new($_);
+### Determine the default charset for this locale:
+my $DefaultCharset;
+{
+    $DefaultCharset = $ENV{LC_CTYPE};
+    $DefaultCharset =~ s/^\w+\.//;  ###  "en_US.ISO8859-1" => "ISO8859-1"
+    $DefaultCharset =~ s/^ISO(\w)/ISO-$1/i;
+    $DefaultCharset ||= "ISO-8859-1";   ### just in case
 }
 
-### UTF-8
-$DecoderFor{'UTF-8'} = MIME::WordDecoder::UTF_8->new();
+### We now pump everything through Unicode by default.
+### However, here's a hook for people who's Unicode support is broken.
+if ($MIME::WordDecoder::NO_UNICODE) { }
+if ($MIME::WordDecoder::NO_UNICODE) {
+    
+    ### Add US-ASCII handler:
+    $DecoderFor{"US-ASCII"} = MIME::WordDecoder::US_ASCII->new;
 
+    ### Add ISO-8859-{1..15} handlers:
+    for (1..15) {
+	$DecoderFor{"ISO-8859-$_"} = MIME::WordDecoder::ISO_8859->new($_);
+    }
+
+    ### Set up default:
+    $Default = $DecoderFor{"ISO-8859-1"};
+}
+else {
+    ### Set up default:
+    $Default = MIME::WordDecoder::Unicode->new($DefaultCharset);
+}
+
+{
+  package main; no strict; local $^W = 0;
+  if (!caller()) { eval join '', <::DATA> or die "$@"; }
+}
 1;           # end the module
 __END__
 
-=head1 SEE ALSO
-
-L<MIME::Tools>
-
 =head1 AUTHOR
 
 Eryq (F<eryq@zeegee.com>), ZeeGee Software Inc (F<http://www.zeegee.com>).
-Dianne Skoll (dianne@skoll.ca)
+
+
+=head1 VERSION
+
+$Revision: 6.111 $ $Date: 2003/06/06 23:41:54 $
 
 =cut
 
-
 BEGIN { unshift @INC, ".", "./etc", "./lib" };
+use MIME::Tools;
 import MIME::WordDecoder;
+print "MIME-tools version = $MIME::Tools::VERSION\n";
+$^W = 1;
 
 ### Decode a MIME string (e.g., into Latin1) via the default decoder:
 my $charset = $ARGV[0] || 'ISO-8859-1';
-my $wd = MIME::WordDecoder->supported($charset) || die "unsupported charset: $charset\n";
+my $wd = MIME::WordDecoder->for($charset) ||
+    die "unsupported charset: $charset\n";
 
-$wd->unknown('#');
-my @encs = (
-	    'ASCII:  =?US-ASCII?Q?Keith_Moore?= <moore@cs.utk.edu>',
-	    'Latin1: =?ISO-8859-1?Q?Keld_J=F8rn_Simonsen?= <keld@dkuug.dk>',
-	    'Latin1: =?ISO-8859-1?Q?Andr=E9_?= Pirard <PIRARD@vm1.ulg.ac.be>',
-	    'Latin1: =?ISO-8859-1?Q?Andr=E9_?=Pirard <PIRARD@vm1.ulg.ac.be>',
-	    ' UTF-8: =?UTF-8?Q?Andr=E9_?=Pirard <PIRARD@vm1.ulg.ac.be>',
-	    'UTF-16: =?UTF-16?Q?=00A=00n=00d=00r=00=E9?= Pirard <PIRARD@vm1.ulg.ac.be>',
-	    ('=?ISO-8859-1?B?SWYgeW91IGNhbiByZWFkIHRoaXMgeW8=?='.
-	     '=?ISO-8859-2?B?dSB1bmRlcnN0YW5kIHRoZSBleGFtcGxlLg==?='.
-	     '=?US-ASCII?Q?.._cool!?='));
-$str = $wd->decode(join "\n", @encs);
-print "$str\n";
+my @encs = 
+    (
+     'ASCII:  =?US-ASCII?Q?Keith_Moore?= <moore@cs.utk.edu>',
+     'BOGUS:  =?BOGUS?B?dSB1bmRlcnN0YW5kIHRoZSBleGFtcGxlLg==?=',
+     'Latin1: =?ISO-8859-1?Q?Keld_J=F8rn_Simonsen?= <keld@dkuug.dk>',
+     'Latin1: =?ISO-8859-1?Q?Andr=E9_?= Pirard <PIRARD@vm1.ulg.ac.be>',
+     'Latin1: =?ISO-8859-1?Q?Andr=E9_?=Pirard <PIRARD@vm1.ulg.ac.be>',
+     'Not-really-UTF-8: =?UTF-8?Q?Andr=E9_?=Pirard <PIRARD@vm1.ulg.ac.be>',
+     'UTF-16: =?UTF-16?Q?=00A=00n=00d=00r=00=E9?= Pirard <PIRARD@vm1.ulg.ac.be>',
+     ('=?US-ASCII?Q?Mishmash:_?=    '.
+	'=?ISO-8859-1?B?SWYgeW91IGNhbiByZWFkIHRoaXMgeW8=?='.
+	'=?ISO-8859-2?B?dSB1bmRlcnN0YW5kIHRoZSBleGFtcGxlLg==?='.
+	'=?US-ASCII?Q?.._cool!?='
+      )
+     );
+foreach (@encs) {
+    $str = $wd->decode($_);
+    print "$str\n\n";    
+}
 1;
